@@ -10,7 +10,7 @@ fn main() {
 	println!("Your UID is {}", current_uid);
 
 	let you = cache.get_user_by_uid(current_uid).expect("No entry for current user!");
-	println!("Your username is {}", you.name());
+	println!("Your username is {}", you.name().to_string_lossy());
 	println!("Your shell is {}", you.shell().display());
 	println!("Your home directory is {}", you.home_dir().display());
 
@@ -20,14 +20,14 @@ fn main() {
     //println!("Your password expiry timestamp is {}", you.password_expire_time());
 
 	let primary_group = cache.get_group_by_gid(you.primary_group_id()).expect("No entry for your primary group!");
-	println!("Your primary group has ID {} and name {}", primary_group.gid(), primary_group.name());
+	println!("Your primary group has ID {} and name {}", primary_group.gid(), primary_group.name().to_string_lossy());
 
 	if primary_group.members().is_empty() {
 		println!("There are no other members of that group.");
 	}
 	else {
 		for username in primary_group.members() {
-			println!("User {} is also a member of that group.", username);
+			println!("User {} is also a member of that group.", username.to_string_lossy());
 		}
 	}
 }
