@@ -29,8 +29,10 @@
 //! best bet is to check for them yourself before passing strings into any
 //! functions.
 
-use std::ffi::{CStr, CString};
+use std::ffi::{CStr, CString, OsStr, OsString};
 use std::fmt;
+use std::io::{Result as IoResult, Error as IoError};
+use std::os::unix::ffi::OsStrExt;
 use std::ptr::read;
 use std::sync::Arc;
 
@@ -46,10 +48,17 @@ use libc::{
     geteuid,
     getgid,
     getegid,
+    getgroups,
+    getgrouplist,
+    c_int,
+};
+#[cfg(not(target_os = "android"))]
+use libc::{
     setpwent,
     getpwent,
     endpwent
 };
+
 use libc::passwd as c_passwd;
 use libc::group as c_group;
 
@@ -60,9 +69,9 @@ pub struct User {
     primary_group: gid_t,
     extras: os::UserExtras,
 
-    /// This user’s name, as an owned `String` possibly shared with a cache.
+    /// This user’s name, as an owned `OsString` possibly shared with a cache.
     /// Prefer using the `name()` accessor to using this field, if possible.
-    pub name_arc: Arc<String>,
+    pub name_arc: Arc<OsString>,
 }
 
 impl User {
@@ -72,28 +81,31 @@ impl User {
     ///
     /// This method does not actually create a new user on the system—it
     /// should only be used for comparing users in tests.
-    pub fn new(uid: uid_t, name: &str, primary_group: gid_t) -> User {
-        User {
-            uid: uid,
-            name_arc: Arc::new(name.to_owned()),
-            primary_group: primary_group,
-            extras: os::UserExtras::default(),
-        }
+    pub fn new<S: AsRef<OsStr> + ?Sized>(uid: uid_t, name: &S, primary_group: gid_t) -> Self {
+        let name_arc = Arc::new(name.into());
+        let extras = os::UserExtras::default();
+    
+        User { uid, name_arc, primary_group, extras }
     }
 
     /// Returns this user’s ID.
     pub fn uid(&self) -> uid_t {
-        self.uid.clone()
+        self.uid
     }
 
     /// Returns this user’s name.
-    pub fn name(&self) -> &str {
+    pub fn name(&self) -> &OsStr {
         &**self.name_arc
     }
 
     /// Returns the ID of this user’s primary group.
     pub fn primary_group_id(&self) -> gid_t {
-        self.primary_group.clone()
+        self.primary_group
+    }
+
+    /// Returns a list of groups this user is a member of
+    pub fn groups(&self) -> Option<Vec<Group>> {
+        get_user_groups(self.name(), self.primary_group_id())
     }
 }
 
@@ -108,7 +120,7 @@ impl fmt::Debug for User {
              .finish()
         }
         else {
-            write!(f, "User({}, {})", self.uid(), self.name())
+            write!(f, "User({}, {})", self.uid(), self.name().to_string_lossy())
         }
     }
 }
@@ -120,9 +132,9 @@ pub struct Group {
     gid: gid_t,
     extras: os::GroupExtras,
 
-    /// This group’s name, as an owned `String` possibly shared with a cache.
+    /// This group’s name, as an owned `OsString` possibly shared with a cache.
     /// Prefer using the `name()` accessor to using this field, if possible.
-    pub name_arc: Arc<String>,
+    pub name_arc: Arc<OsString>,
 }
 
 impl Group {
@@ -132,21 +144,20 @@ impl Group {
     ///
     /// This method does not actually create a new group on the system—it
     /// should only be used for comparing groups in tests.
-    pub fn new(gid: gid_t, name: &str) -> Self {
-        Group {
-            gid: gid,
-            name_arc: Arc::new(String::from(name)),
-            extras: os::GroupExtras::default(),
-        }
+    pub fn new<S: AsRef<OsStr> + ?Sized>(gid: gid_t, name: &S) -> Self {
+        let name_arc = Arc::new(name.into());
+        let extras = os::GroupExtras::default();
+    
+        Group { gid, name_arc, extras }
     }
 
     /// Returns this group’s ID.
     pub fn gid(&self) -> gid_t {
-        self.gid.clone()
+        self.gid
     }
 
-    /// Returns this group's name.
-    pub fn name(&self) -> &str {
+    /// Returns this group’s name.
+    pub fn name(&self) -> &OsStr {
         &**self.name_arc
     }
 }
@@ -161,39 +172,23 @@ impl fmt::Debug for Group {
              .finish()
         }
         else {
-            write!(f, "Group({}, {})", self.gid(), self.name())
+            write!(f, "Group({}, {})", self.gid(), self.name().to_string_lossy())
         }
     }
 }
 
 
-/// Reads data from a `*char` field in `c_passwd` or `g_group` into a UTF-8
-/// `String` for use in a user or group value.
+/// Reads data from a `*char` field in `c_passwd` or `g_group` into an
+/// `OsString` for use in a user or group value.
 ///
-/// Although `from_utf8_lossy` returns a clone-on-write string, we immediately
-/// clone it anyway: the underlying buffer is managed by the C library, not by
-/// us, so we *need* to move data out of it before the next user gets read.
-unsafe fn from_raw_buf(p: *const c_char) -> String {
-    CStr::from_ptr(p).to_string_lossy().into_owned()
-}
-
-/// Converts a raw pointer, which could be null, into a safe reference that
-/// might be `None` instead.
-///
-/// This is basically the unstable `ptr_as_ref` feature:
-/// https://github.com/rust-lang/rust/issues/27780
-/// When that stabilises, this can be replaced.
-unsafe fn ptr_as_ref<T>(pointer: *const T) -> Option<T> {
-    if pointer.is_null() {
-        None
-    }
-    else {
-        Some(read(pointer))
-    }
+/// The underlying buffer is managed by the C library, not by us, so we *need*
+/// to move data out of it before the next user gets read.
+unsafe fn from_raw_buf(p: *const c_char) -> OsString {
+    OsStr::from_bytes(CStr::from_ptr(p).to_bytes()).to_os_string()
 }
 
 unsafe fn passwd_to_user(pointer: *const c_passwd) -> Option<User> {
-    if let Some(passwd) = ptr_as_ref(pointer) {
+    if let Some(passwd) = pointer.as_ref().map(|p| read(p)) {
         let name = Arc::new(from_raw_buf(passwd.pw_name));
 
         Some(User {
@@ -209,7 +204,7 @@ unsafe fn passwd_to_user(pointer: *const c_passwd) -> Option<User> {
 }
 
 unsafe fn struct_to_group(pointer: *const c_group) -> Option<Group> {
-    if let Some(group) = ptr_as_ref(pointer) {
+    if let Some(group) = pointer.as_ref().map(|p| read(p)) {
         let name = Arc::new(from_raw_buf(group.gr_name));
 
         Some(Group {
@@ -227,10 +222,10 @@ unsafe fn struct_to_group(pointer: *const c_group) -> Option<Group> {
 ///
 /// The list of members is, in true C fashion, a pointer to a pointer of
 /// characters, terminated by a null pointer. We check `members[0]`, then
-/// `members[1]`, and so on, until that null pointer is reached. It doesn't
+/// `members[1]`, and so on, until that null pointer is reached. It doesn’t
 /// specify whether we should expect a null pointer or a pointer to a null
 /// pointer, so we check for both here!
-unsafe fn members(groups: *mut *mut c_char) -> Vec<String> {
+unsafe fn members(groups: *mut *mut c_char) -> Vec<OsString> {
     let mut members = Vec::new();
 
     for i in 0.. {
@@ -259,8 +254,8 @@ pub fn get_user_by_uid(uid: uid_t) -> Option<User> {
 
 /// Searches for a `User` with the given username in the system’s user database.
 /// Returns it if one is found, otherwise returns `None`.
-pub fn get_user_by_name(username: &str) -> Option<User> {
-    if let Ok(username) = CString::new(username) {
+pub fn get_user_by_name<S: AsRef<OsStr> + ?Sized>(username: &S) -> Option<User> {
+    if let Ok(username) = CString::new(username.as_ref().as_bytes()) {
         unsafe {
             let passwd = getpwnam(username.as_ptr());
             passwd_to_user(passwd)
@@ -285,8 +280,8 @@ pub fn get_group_by_gid(gid: gid_t) -> Option<Group> {
 
 /// Searches for a `Group` with the given group name in the system’s group database.
 /// Returns it if one is found, otherwise returns `None`.
-pub fn get_group_by_name(group_name: &str) -> Option<Group> {
-    if let Ok(group_name) = CString::new(group_name) {
+pub fn get_group_by_name<S: AsRef<OsStr> + ?Sized>(group_name: &S) -> Option<Group> {
+    if let Ok(group_name) = CString::new(group_name.as_ref().as_bytes()) {
         unsafe {
             let group = getgrnam(group_name.as_ptr());
             struct_to_group(group)
@@ -306,7 +301,7 @@ pub fn get_current_uid() -> uid_t {
 }
 
 /// Returns the username of the user running the process.
-pub fn get_current_username() -> Option<String> {
+pub fn get_current_username() -> Option<OsString> {
     let uid = get_current_uid();
     get_user_by_uid(uid).map(|u| Arc::try_unwrap(u.name_arc).unwrap())
 }
@@ -317,7 +312,7 @@ pub fn get_effective_uid() -> uid_t {
 }
 
 /// Returns the username of the effective user running the process.
-pub fn get_effective_username() -> Option<String> {
+pub fn get_effective_username() -> Option<OsString> {
     let uid = get_effective_uid();
     get_user_by_uid(uid).map(|u| Arc::try_unwrap(u.name_arc).unwrap())
 }
@@ -328,7 +323,7 @@ pub fn get_current_gid() -> gid_t {
 }
 
 /// Returns the groupname of the user running the process.
-pub fn get_current_groupname() -> Option<String> {
+pub fn get_current_groupname() -> Option<OsString> {
     let gid = get_current_gid();
     get_group_by_gid(gid).map(|g| Arc::try_unwrap(g.name_arc).unwrap())
 }
@@ -339,48 +334,99 @@ pub fn get_effective_gid() -> gid_t {
 }
 
 /// Returns the groupname of the effective user running the process.
-pub fn get_effective_groupname() -> Option<String> {
+pub fn get_effective_groupname() -> Option<OsString> {
     let gid = get_effective_gid();
     get_group_by_gid(gid).map(|g| Arc::try_unwrap(g.name_arc).unwrap())
 }
 
+/// Returns the group access list for the current process.
+pub fn group_access_list() -> IoResult<Vec<Group>> {
+    let mut buff: Vec<gid_t> = vec![0; 1024];
+    
+    let res = unsafe {
+        getgroups(1024, buff.as_mut_ptr())
+    };
+
+    if res < 0 {
+        Err(IoError::last_os_error())
+    }
+    else {
+        let mut groups = buff.into_iter()
+                                     .filter_map(|i| get_group_by_gid(i))
+                                     .collect::<Vec<_>>();
+        groups.dedup_by_key(|i| i.gid());
+        Ok(groups)
+    }
+}
+
+/// Returns groups for a provided user name and primary group id
+pub fn get_user_groups<S: AsRef<OsStr> + ?Sized>(username: &S, gid: gid_t) -> Option<Vec<Group>> {
+    // MacOS uses i32 instead of gid_t in getgrouplist for unknown reasons
+    #[cfg(all(unix, target_os="macos"))]
+    let mut buff: Vec<i32> = vec![0; 1024];
+    #[cfg(all(unix, not(target_os="macos")))]
+    let mut buff: Vec<gid_t> = vec![0; 1024];
+
+    let name = CString::new(username.as_ref().as_bytes()).unwrap();
+    let mut count = buff.len() as c_int;
+
+    // MacOS uses i32 instead of gid_t in getgrouplist for unknown reasons
+    #[cfg(all(unix, target_os="macos"))]
+    let res = unsafe {
+        getgrouplist(name.as_ptr(), gid as i32, buff.as_mut_ptr(), &mut count)
+    };
+    
+    #[cfg(all(unix, not(target_os="macos")))]
+    let res = unsafe {
+        getgrouplist(name.as_ptr(), gid, buff.as_mut_ptr(), &mut count)
+    };
+
+    if res < 0 {
+        None
+    }
+    else {
+        buff.dedup();
+        buff.into_iter()
+            .filter_map(|i| get_group_by_gid(i as gid_t))
+            .collect::<Vec<_>>()
+            .into()
+    }
+}
+
+
 
 /// An iterator over every user present on the system.
+struct AllUsers;
+
+/// Creates a new iterator over every user present on the system.
 ///
-/// This struct actually requires no fields, but has one hidden one to make it
-/// `unsafe` to create.
-pub struct AllUsers(());
-
-impl AllUsers {
-
-    /// Creates a new iterator over every user present on the system.
-    ///
-    /// ## Unsafety
-    ///
-    /// This constructor is marked as `unsafe`, which is odd for a crate
-    /// that's meant to be a safe interface. It *has* to be unsafe because
-    /// we cannot guarantee that the underlying C functions,
-    /// `getpwent`/`setpwent`/`endpwent` that iterate over the system's
-    /// `passwd` entries, are called in a thread-safe manner.
-    ///
-    /// These functions [modify a global
-    /// state](http://man7.org/linux/man-pages/man3/getpwent.3.html#ATTRIBUTES),
-    /// and if any are used at the same time, the state could be reset,
-    /// resulting in a data race. We cannot even place it behind an internal
-    /// `Mutex`, as there is nothing stopping another `extern` function
-    /// definition from calling it!
-    ///
-    /// So to iterate all users, construct the iterator inside an `unsafe`
-    /// block, then make sure to not make a new instance of it until
-    /// iteration is over.
-    pub unsafe fn new() -> AllUsers {
-        setpwent();
-        AllUsers(())
-    }
+/// ## Unsafety
+///
+/// This constructor is marked as `unsafe`, which is odd for a crate
+/// that’s meant to be a safe interface. It *has* to be unsafe because
+/// we cannot guarantee that the underlying C functions,
+/// `getpwent`/`setpwent`/`endpwent` that iterate over the system’s
+/// `passwd` entries, are called in a thread-safe manner.
+///
+/// These functions [modify a global
+/// state](http://man7.org/linux/man-pages/man3/getpwent.3.html#ATTRIBUTES),
+/// and if any are used at the same time, the state could be reset,
+/// resulting in a data race. We cannot even place it behind an internal
+/// `Mutex`, as there is nothing stopping another `extern` function
+/// definition from calling it!
+///
+/// So to iterate all users, construct the iterator inside an `unsafe`
+/// block, then make sure to not make a new instance of it until
+/// iteration is over.
+pub unsafe fn all_users() -> impl Iterator<Item=User> {
+    #[cfg(not(target_os = "android"))]
+    setpwent();
+    AllUsers
 }
 
 impl Drop for AllUsers {
     fn drop(&mut self) {
+        #[cfg(not(target_os = "android"))]
         unsafe { endpwent() };
     }
 }
@@ -388,6 +434,11 @@ impl Drop for AllUsers {
 impl Iterator for AllUsers {
     type Item = User;
 
+    #[cfg(target_os = "android")]
+    fn next(&mut self) -> Option<User> {
+        None
+    }
+    #[cfg(not(target_os = "android"))]
     fn next(&mut self) -> Option<User> {
         unsafe { passwd_to_user(getpwent()) }
     }
@@ -417,9 +468,10 @@ pub mod os {
     /// Although the `passwd` struct is common among Unix systems, its actual
     /// format can vary. See the definitions in the `base` module to check which
     /// fields are actually present.
-    #[cfg(any(target_os = "linux", target_os = "macos", target_os = "freebsd", target_os = "dragonfly", target_os = "openbsd", target_os = "netbsd"))]
+    #[cfg(any(target_os = "linux", target_os = "android", target_os = "macos", target_os = "freebsd", target_os = "dragonfly", target_os = "openbsd", target_os = "netbsd", target_os = "solaris"))]
     pub mod unix {
-        use std::path::Path;
+        use std::ffi::{OsStr, OsString};
+        use std::path::{Path, PathBuf};
 
         use super::super::{c_passwd, c_group, members, from_raw_buf, Group};
 
@@ -432,7 +484,7 @@ pub mod os {
             /// Sets this user value’s home directory to the given string.
             /// Can be used to construct test users, which by default come with a
             /// dummy home directory string.
-            fn with_home_dir(self, home_dir: &str) -> Self;
+            fn with_home_dir<S: AsRef<OsStr> + ?Sized>(self, home_dir: &S) -> Self;
 
             /// Returns a path to this user’s shell.
             fn shell(&self) -> &Path;
@@ -440,18 +492,15 @@ pub mod os {
             /// Sets this user’s shell path to the given string.
             /// Can be used to construct test users, which by default come with a
             /// dummy shell field.
-            fn with_shell(self, shell: &str) -> Self;
+            fn with_shell<S: AsRef<OsStr> + ?Sized>(self, shell: &S) -> Self;
 
-            /// Returns the user's encrypted password.
-            fn password(&self) -> &str;
+            /// Returns the user’s encrypted password.
+            fn password(&self) -> &OsStr;
 
-            /// Sets this user's password to the given string.
+            /// Sets this user’s password to the given string.
             /// Can be used to construct tests users, which by default come with a
             /// dummy password field.
-            fn with_password(self, password: &str) -> Self;
-
-            // TODO(ogham): Isn’t it weird that the setters take string slices, but
-            // the getters return paths?
+            fn with_password<S: AsRef<OsStr> + ?Sized>(self, password: &S) -> Self;
         }
 
         /// Unix-specific extensions for `Group`s.
@@ -459,10 +508,10 @@ pub mod os {
 
             /// Returns a slice of the list of users that are in this group as
             /// their non-primary group.
-            fn members(&self) -> &[String];
+            fn members(&self) -> &[OsString];
 
             /// Adds a new member to this group.
-            fn add_member(self, name: &str) -> Self;
+            fn add_member<S: AsRef<OsStr> + ?Sized>(self, name: &S) -> Self;
         }
 
         /// Unix-specific fields for `User`s.
@@ -470,21 +519,21 @@ pub mod os {
         pub struct UserExtras {
 
             /// The path to the user’s home directory.
-            pub home_dir: String,
+            pub home_dir: PathBuf,
 
             /// The path to the user’s shell.
-            pub shell: String,
+            pub shell: PathBuf,
 
-            /// The user's encrypted password.
-            pub password: String,
+            /// The user’s encrypted password.
+            pub password: OsString,
         }
 
         impl Default for UserExtras {
-            fn default() -> UserExtras {
+            fn default() -> Self {
                 UserExtras {
-                    home_dir: String::from("/var/empty"),
-                    shell:    String::from("/bin/false"),
-                    password: String::from("*"),
+                    home_dir: "/var/empty".into(),
+                    shell:    "/bin/false".into(),
+                    password: "*".into(),
                 }
             }
         }
@@ -492,30 +541,33 @@ pub mod os {
         impl UserExtras {
             /// Extract the OS-specific fields from the C `passwd` struct that
             /// we just read.
-            pub unsafe fn from_passwd(passwd: c_passwd) -> UserExtras {
-                let home_dir = from_raw_buf(passwd.pw_dir);
-                let shell    = from_raw_buf(passwd.pw_shell);
-                let password = from_raw_buf(passwd.pw_passwd);
+            pub unsafe fn from_passwd(passwd: c_passwd) -> Self {
+                #[cfg(target_os = "android")]
+                {
+                    Default::default()
+                }
+                #[cfg(not(target_os = "android"))]
+                {
+                    let home_dir = from_raw_buf(passwd.pw_dir).into();
+                    let shell    = from_raw_buf(passwd.pw_shell).into();
+                    let password = from_raw_buf(passwd.pw_passwd);
 
-                UserExtras {
-                    home_dir:  home_dir,
-                    shell:     shell,
-                    password:  password,
+                    UserExtras { home_dir, shell, password }
                 }
             }
         }
 
-        #[cfg(any(target_os = "linux"))]
+        #[cfg(any(target_os = "linux", target_os = "android", target_os = "solaris"))]
         use super::super::User;
 
-        #[cfg(any(target_os = "linux"))]
+        #[cfg(any(target_os = "linux", target_os = "android", target_os = "solaris"))]
         impl UserExt for User {
             fn home_dir(&self) -> &Path {
                 Path::new(&self.extras.home_dir)
             }
 
-            fn with_home_dir(mut self, home_dir: &str) -> User {
-                self.extras.home_dir = home_dir.to_owned();
+            fn with_home_dir<S: AsRef<OsStr> + ?Sized>(mut self, home_dir: &S) -> Self {
+                self.extras.home_dir = home_dir.into();
                 self
             }
 
@@ -523,17 +575,17 @@ pub mod os {
                 Path::new(&self.extras.shell)
             }
 
-            fn with_shell(mut self, shell: &str) -> User {
-                self.extras.shell = shell.to_owned();
+            fn with_shell<S: AsRef<OsStr> + ?Sized>(mut self, shell: &S) -> Self {
+                self.extras.shell = shell.into();
                 self
             }
 
-            fn password(&self) -> &str {
+            fn password(&self) -> &OsStr {
                 &self.extras.password
             }
 
-            fn with_password(mut self, password: &str) -> User {
-                self.extras.password = password.to_owned();
+            fn with_password<S: AsRef<OsStr> + ?Sized>(mut self, password: &S) -> Self {
+                self.extras.password = password.into();
                 self
             }
         }
@@ -543,28 +595,24 @@ pub mod os {
         pub struct GroupExtras {
 
             /// Vector of usernames that are members of this group.
-            pub members: Vec<String>,
+            pub members: Vec<OsString>,
         }
 
         impl GroupExtras {
             /// Extract the OS-specific fields from the C `group` struct that
             /// we just read.
-            pub unsafe fn from_struct(group: c_group) -> GroupExtras {
-                let members = members(group.gr_mem);
-
-                GroupExtras {
-                    members: members,
-                }
+            pub unsafe fn from_struct(group: c_group) -> Self {
+                GroupExtras { members: members(group.gr_mem) }
             }
         }
 
         impl GroupExt for Group {
-            fn members(&self) -> &[String] {
+            fn members(&self) -> &[OsString] {
                 &*self.extras.members
             }
 
-            fn add_member(mut self, member: &str) -> Group {
-                self.extras.members.push(member.to_owned());
+            fn add_member<S: AsRef<OsStr> + ?Sized>(mut self, member: &S) -> Self {
+                self.extras.members.push(member.into());
                 self
             }
         }
@@ -576,6 +624,7 @@ pub mod os {
     /// C structs.
     #[cfg(any(target_os = "macos", target_os = "freebsd", target_os = "dragonfly", target_os = "openbsd", target_os = "netbsd"))]
     pub mod bsd {
+        use std::ffi::OsStr;
         use std::path::Path;
         use libc::time_t;
         use super::super::{c_passwd, User};
@@ -598,7 +647,7 @@ pub mod os {
         impl UserExtras {
             /// Extract the OS-specific fields from the C `passwd` struct that
             /// we just read.
-            pub unsafe fn from_passwd(passwd: c_passwd) -> UserExtras {
+            pub unsafe fn from_passwd(passwd: c_passwd) -> Self {
                 UserExtras {
                     change: passwd.pw_change,
                     expire: passwd.pw_expire,
@@ -612,8 +661,8 @@ pub mod os {
                 Path::new(&self.extras.extras.home_dir)
             }
 
-            fn with_home_dir(mut self, home_dir: &str) -> User {
-                self.extras.extras.home_dir = home_dir.to_owned();
+            fn with_home_dir<S: AsRef<OsStr> + ?Sized>(mut self, home_dir: &S) -> Self {
+                self.extras.extras.home_dir = home_dir.into();
                 self
             }
 
@@ -621,17 +670,17 @@ pub mod os {
                 Path::new(&self.extras.extras.shell)
             }
 
-            fn with_shell(mut self, shell: &str) -> User {
-                self.extras.extras.shell = shell.to_owned();
+            fn with_shell<S: AsRef<OsStr> + ?Sized>(mut self, shell: &S) -> Self {
+                self.extras.extras.shell = shell.into();
                 self
             }
 
-            fn password(&self) -> &str {
+            fn password(&self) -> &OsStr {
                 &self.extras.extras.password
             }
 
-            fn with_password(mut self, password: &str) -> User {
-                self.extras.extras.password = password.to_owned();
+            fn with_password<S: AsRef<OsStr> + ?Sized>(mut self, password: &S) -> Self {
+                self.extras.extras.password = password.into();
                 self
             }
         }
@@ -639,25 +688,25 @@ pub mod os {
         /// BSD-specific accessors for `User`s.
         pub trait UserExt {
 
-            /// Returns this user's password change timestamp.
+            /// Returns this user’s password change timestamp.
             fn password_change_time(&self) -> time_t;
 
-            /// Returns this user's password expiry timestamp.
+            /// Returns this user’s password expiry timestamp.
             fn password_expire_time(&self) -> time_t;
         }
 
         impl UserExt for User {
             fn password_change_time(&self) -> time_t {
-                self.extras.change.clone()
+                self.extras.change
             }
 
             fn password_expire_time(&self) -> time_t {
-                self.extras.expire.clone()
+                self.extras.expire
             }
         }
 
         impl Default for UserExtras {
-            fn default() -> UserExtras {
+            fn default() -> Self {
                 UserExtras {
                     extras: super::unix::UserExtras::default(),
                     change: 0,
@@ -672,11 +721,11 @@ pub mod os {
     pub type UserExtras = bsd::UserExtras;
 
     /// Any extra fields on a `User` specific to the current platform.
-    #[cfg(any(target_os = "linux"))]
+    #[cfg(any(target_os = "linux", target_os = "android", target_os = "solaris"))]
     pub type UserExtras = unix::UserExtras;
 
     /// Any extra fields on a `Group` specific to the current platform.
-    #[cfg(any(target_os = "linux", target_os = "macos", target_os = "freebsd", target_os = "dragonfly", target_os = "openbsd", target_os = "netbsd"))]
+    #[cfg(any(target_os = "linux", target_os = "android", target_os = "macos", target_os = "freebsd", target_os = "dragonfly", target_os = "openbsd", target_os = "netbsd", target_os = "solaris"))]
     pub type GroupExtras = unix::GroupExtras;
 }
 
@@ -737,6 +786,15 @@ mod test {
         assert!(user.is_none());
     }
 
+    #[test]
+    fn user_get_groups() {
+        let uid = get_current_uid();
+        let user = get_user_by_uid(uid).unwrap();
+        let groups = user.groups().unwrap();
+        println!("Groups: {:?}", groups);
+        assert!(groups.len() > 0);
+    }
+
     #[test]
     fn group_by_name() {
         // We cannot really test for arbitrary groups as they might not exist on the machine
